@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -34,6 +34,7 @@ void *pdf_find_item(fz_context *ctx, fz_store_drop_fn *drop, pdf_obj *key);
 void pdf_remove_item(fz_context *ctx, fz_store_drop_fn *drop, pdf_obj *key);
 void pdf_empty_store(fz_context *ctx, pdf_document *doc);
 void pdf_purge_locals_from_store(fz_context *ctx, pdf_document *doc);
+void pdf_purge_object_from_store(fz_context *ctx, pdf_document *doc, int num);
 
 /*
  * Structures used for managing resource locations and avoiding multiple
@@ -46,7 +47,7 @@ void pdf_purge_locals_from_store(fz_context *ctx, pdf_document *doc);
 enum { PDF_SIMPLE_FONT_RESOURCE=1, PDF_CID_FONT_RESOURCE=2, PDF_CJK_FONT_RESOURCE=3 };
 enum { PDF_SIMPLE_ENCODING_LATIN, PDF_SIMPLE_ENCODING_GREEK, PDF_SIMPLE_ENCODING_CYRILLIC };
 
-/* The contents of this structure are defined publically just so we can
+/* The contents of this structure are defined publicly just so we can
  * define this on the stack. */
 typedef struct
 {
@@ -56,10 +57,18 @@ typedef struct
 	int local_xref;
 } pdf_font_resource_key;
 
+typedef struct
+{
+	unsigned char digest[16];
+	int local_xref;
+} pdf_colorspace_resource_key;
+
 pdf_obj *pdf_find_font_resource(fz_context *ctx, pdf_document *doc, int type, int encoding, fz_font *item, pdf_font_resource_key *key);
 pdf_obj *pdf_insert_font_resource(fz_context *ctx, pdf_document *doc, pdf_font_resource_key *key, pdf_obj *obj);
+pdf_obj *pdf_find_colorspace_resource(fz_context *ctx, pdf_document *doc, fz_colorspace *item, pdf_colorspace_resource_key *key);
+pdf_obj *pdf_insert_colorspace_resource(fz_context *ctx, pdf_document *doc, pdf_colorspace_resource_key *key, pdf_obj *obj);
 void pdf_drop_resource_tables(fz_context *ctx, pdf_document *doc);
-void pdf_purge_local_font_resources(fz_context *ctx, pdf_document *doc);
+void pdf_purge_local_resources(fz_context *ctx, pdf_document *doc);
 
 typedef struct pdf_function pdf_function;
 
@@ -101,6 +110,8 @@ fz_image *pdf_load_image(fz_context *ctx, pdf_document *doc, pdf_obj *obj);
 
 pdf_obj *pdf_add_image(fz_context *ctx, pdf_document *doc, fz_image *image);
 
+pdf_obj *pdf_add_colorspace(fz_context *ctx, pdf_document *doc, fz_colorspace *cs);
+
 typedef struct
 {
 	fz_storable storable;
