@@ -1,9 +1,10 @@
 /*
 Multi-threaded rendering of all pages in a document to PNG images.
 
-First look at doc/example.c and make sure you understand it.
-Then read the multi-threading section in doc/overview.txt,
-before coming back here to see an example of multi-threading.
+First look at docs/example.c and make sure you understand it.
+Then, before coming back here to see an example of multi-threading,
+please read the multi-threading section in:
+https://mupdf.readthedocs.io/en/latest/using-mupdf.html#multi-threading
 
 This example will create one main thread for reading pages from the
 document, and one thread per page for rendering. After rendering
@@ -40,7 +41,7 @@ depending on your environment.
 // A convenience function for dying abruptly on pthread errors.
 
 void
-fail(char *msg)
+fail(const char *msg)
 {
 	fprintf(stderr, "%s\n", msg);
 	abort();
@@ -49,7 +50,7 @@ fail(char *msg)
 // The data structure passed between the requesting main thread and
 // each rendering thread.
 
-struct data {
+struct thread_data {
 	// A pointer to the original context in the main thread sent
 	// from main to rendering thread. It will be used to create
 	// each rendering thread's context clone.
@@ -74,6 +75,9 @@ struct data {
 	// thread, and then back from the rendering thread to the main
 	// thread.
 	fz_pixmap *pix;
+
+	// This is a note of whether a given thread failed or not.
+	int failed;
 };
 
 // This is the function run by each rendering function. It takes
@@ -81,34 +85,45 @@ struct data {
 // renders the display list into the pixmap before exiting.
 
 void *
-renderer(void *data)
+renderer(void *data_)
 {
-	int pagenumber = ((struct data *) data)->pagenumber;
-	fz_context *ctx = ((struct data *) data)->ctx;
-	fz_display_list *list = ((struct data *) data)->list;
-	fz_rect bbox = ((struct data *) data)->bbox;
-	fz_pixmap *pix = ((struct data *) data)->pix;
-	fz_device *dev;
+	struct thread_data *data = (struct thread_data *)data_;
+	int pagenumber = data->pagenumber;
+	fz_context *ctx = data->ctx;
+	fz_display_list *list = data->list;
+	fz_rect bbox = data->bbox;
+	fz_device *dev = NULL;
 
 	fprintf(stderr, "thread at page %d loading!\n", pagenumber);
 
 	// The context pointer is pointing to the main thread's
 	// context, so here we create a new context based on it for
 	// use in this thread.
-
 	ctx = fz_clone_context(ctx);
 
 	// Next we run the display list through the draw device which
 	// will render the request area of the page to the pixmap.
 
+	fz_var(dev);
+
 	fprintf(stderr, "thread at page %d rendering!\n", pagenumber);
-	dev = fz_new_draw_device(ctx, fz_identity, pix);
-	fz_run_display_list(ctx, list, dev, fz_identity, bbox, NULL);
-	fz_close_device(ctx, dev);
-	fz_drop_device(ctx, dev);
+	fz_try(ctx)
+	{
+		// Create a white pixmap using the correct dimensions.
+		data->pix = fz_new_pixmap_with_bbox(ctx, fz_device_rgb(ctx), fz_round_rect(bbox), NULL, 0);
+		fz_clear_pixmap_with_value(ctx, data->pix, 0xff);
 
-	// This threads context is freed.
+		// Do the actual rendering.
+		dev = fz_new_draw_device(ctx, fz_identity, data->pix);
+		fz_run_display_list(ctx, list, dev, fz_identity, bbox, NULL);
+		fz_close_device(ctx, dev);
+	}
+	fz_always(ctx)
+		fz_drop_device(ctx, dev);
+	fz_catch(ctx)
+		data->failed = 1;
 
+	// Free this thread's context.
 	fz_drop_context(ctx);
 
 	fprintf(stderr, "thread at page %d done!\n", pagenumber);
@@ -144,12 +159,11 @@ int main(int argc, char **argv)
 	fz_locks_context locks;
 	pthread_mutex_t mutex[FZ_LOCK_MAX];
 	fz_context *ctx;
-	fz_document *doc;
+	fz_document *doc = NULL;
 	int threads;
 	int i;
 
 	// Initialize FZ_LOCK_MAX number of non-recursive mutexes.
-
 	for (i = 0; i < FZ_LOCK_MAX; i++)
 	{
 		if (pthread_mutex_init(&mutex[i], NULL) != 0)
@@ -161,136 +175,148 @@ int main(int argc, char **argv)
 	// the user data is a pointer to the array of mutexes so the
 	// locking functions can find the relevant lock to change when
 	// they are called. This way we avoid global variables.
-
 	locks.user = mutex;
 	locks.lock = lock_mutex;
 	locks.unlock = unlock_mutex;
 
-	// This is the main threads context function, so supply the
+	// This is the main thread's context function, so supply the
 	// locking structure. This context will be used to parse all
 	// the pages from the document.
-
 	ctx = fz_new_context(NULL, &locks, FZ_STORE_UNLIMITED);
 
-	// Register default file types.
+	fz_var(thread);
+	fz_var(doc);
 
-	fz_register_document_handlers(ctx);
-
-	// Open the PDF, XPS or CBZ document. Note, this binds doc to ctx.
-	// You must only ever use doc with ctx - never a clone of it!
-
-	doc = fz_open_document(ctx, filename);
-
-	// Retrieve the number of pages, which translates to the
-	// number of threads used for rendering pages.
-
-	threads = fz_count_pages(ctx, doc);
-	fprintf(stderr, "spawning %d threads, one per page...\n", threads);
-
-	thread = malloc(threads * sizeof (pthread_t));
-
-	for (i = 0; i < threads; i++)
+	fz_try(ctx)
 	{
-		fz_page *page;
-		fz_rect bbox;
-		fz_display_list *list;
-		fz_device *dev;
-		fz_pixmap *pix;
-		struct data *data;
+		// Register default file types.
+		fz_register_document_handlers(ctx);
 
-		// Load the relevant page for each thread. Note, that this
-		// cannot be done on the worker threads, as each use of doc
-		// uses ctx, and only one thread can be using ctx at a time.
+		// Open the PDF, XPS or CBZ document.
+		doc = fz_open_document(ctx, filename);
 
-		page = fz_load_page(ctx, doc, i);
+		// Retrieve the number of pages, which translates to the
+		// number of threads used for rendering pages.
+		threads = fz_count_pages(ctx, doc);
+		fprintf(stderr, "spawning %d threads, one per page...\n", threads);
 
-		// Compute the bounding box for each page.
+		thread = malloc(threads * sizeof (*thread));
 
-		bbox = fz_bound_page(ctx, page);
+		for (i = 0; i < threads; i++)
+		{
+			fz_page *page;
+			fz_rect bbox;
+			fz_display_list *list;
+			fz_device *dev = NULL;
+			fz_pixmap *pix;
+			struct thread_data *data;
 
-		// Create a display list that will hold the drawing
-		// commands for the page. Once we have the display list
-		// this can safely be used on any other thread as it is
-		// not bound to a given context.
+			fz_var(dev);
 
-		list = fz_new_display_list(ctx, bbox);
+			fz_try(ctx)
+			{
+				// Load the relevant page for each thread. Note, that this
+				// cannot be done on the worker threads, as only one thread
+				// at a time can ever be accessing the document.
+				page = fz_load_page(ctx, doc, i);
 
-		// Run the loaded page through a display list device
-		// to populate the page's display list.
+				// Compute the bounding box for each page.
+				bbox = fz_bound_page(ctx, page);
 
-		dev = fz_new_list_device(ctx, list);
-		fz_run_page(ctx, page, dev, fz_identity, NULL);
-		fz_close_device(ctx, dev);
-		fz_drop_device(ctx, dev);
+				// Create a display list that will hold the drawing
+				// commands for the page. Once we have the display list
+				// this can safely be used on any other thread.
+				list = fz_new_display_list(ctx, bbox);
 
-		// The page is no longer needed, all drawing commands
-		// are now in the display list.
+				// Create a display list device to populate the page's display list.
+				dev = fz_new_list_device(ctx, list);
 
-		fz_drop_page(ctx, page);
+				// Run the page to that device.
+				fz_run_page(ctx, page, dev, fz_identity, NULL);
 
-		// Create a white pixmap using the correct dimensions.
+				// Close the device neatly, so everything is flushed to the list.
+				fz_close_device(ctx, dev);
+			}
+			fz_always(ctx)
+			{
+				// Throw away the device.
+				fz_drop_device(ctx, dev);
 
-		pix = fz_new_pixmap_with_bbox(ctx, fz_device_rgb(ctx), fz_round_rect(bbox), NULL, 0);
-		fz_clear_pixmap_with_value(ctx, pix, 0xff);
+				// The page is no longer needed, all drawing commands
+				// are now in the display list.
+				fz_drop_page(ctx, page);
+			}
+			fz_catch(ctx)
+				fz_rethrow(ctx);
 
-		// Populate the data structure to be sent to the
-		// rendering thread for this page.
+			// Populate the data structure to be sent to the
+			// rendering thread for this page.
+			data = malloc(sizeof (*data));
 
-		data = malloc(sizeof (struct data));
+			data->pagenumber = i + 1;
+			data->ctx = ctx;
+			data->list = list;
+			data->bbox = bbox;
+			data->pix = NULL;
+			data->failed = 0;
 
-		data->pagenumber = i + 1;
-		data->ctx = ctx;
-		data->list = list;
-		data->bbox = bbox;
-		data->pix = pix;
+			// Create the thread and pass it the data structure.
+			if (pthread_create(&thread[i], NULL, renderer, data) != 0)
+				fail("pthread_create()");
+		}
 
-		// Create the thread and pass it the data structure.
+		// Now each thread is rendering pages, so wait for each thread
+		// to complete its rendering.
+		fprintf(stderr, "joining %d threads...\n", threads);
+		for (i = 0; i < threads; i++)
+		{
+			char filename[42];
+			struct thread_data *data;
 
-		if (pthread_create(&thread[i], NULL, renderer, data) != 0)
-			fail("pthread_create()");
+			if (pthread_join(thread[i], (void **) &data) != 0)
+				fail("pthread_join");
+
+			if (data->failed)
+			{
+				fprintf(stderr, "\tRendering for page %d failed\n", i + 1);
+			}
+			else
+			{
+				sprintf(filename, "out%04d.png", i);
+				fprintf(stderr, "\tSaving %s...\n", filename);
+
+				// Write the rendered image to a PNG file
+				fz_save_pixmap_as_png(ctx, data->pix, filename);
+			}
+
+			// Free the thread's pixmap and display list.
+			fz_drop_pixmap(ctx, data->pix);
+			fz_drop_display_list(ctx, data->list);
+
+			// Free the data structure passed back and forth
+			// between the main thread and rendering thread.
+			free(data);
+		}
+	}
+	fz_always(ctx)
+	{
+		// Free the thread structure
+		free(thread);
+
+		// Drop the document
+		fz_drop_document(ctx, doc);
+	}
+	fz_catch(ctx)
+	{
+		fz_report_error(ctx);
+		fail("error");
 	}
 
-	// Now each thread is rendering pages, so wait for each thread
-	// to complete its rendering.
-
-	fprintf(stderr, "joining %d threads...\n", threads);
-	for (i = threads - 1; i >= 0; i--)
-	{
-		char filename[42];
-		struct data *data;
-
-		if (pthread_join(thread[i], (void **) &data) != 0)
-			fail("pthread_join");
-
-		sprintf(filename, "out%04d.png", i);
-		fprintf(stderr, "\tSaving %s...\n", filename);
-
-		// Write the rendered image to a PNG file
-
-		fz_save_pixmap_as_png(ctx, data->pix, filename);
-
-		// Free the thread's pixmap and display list since
-		// they were allocated by the main thread above.
-
-		fz_drop_pixmap(ctx, data->pix);
-		fz_drop_display_list(ctx, data->list);
-
-		// Free the data structured passed back and forth
-		// between the main thread and rendering thread.
-
-		free(data);
-	}
+	// Finally the main thread's context is freed.
+	fz_drop_context(ctx);
 
 	fprintf(stderr, "finally!\n");
 	fflush(NULL);
 
-	free(thread);
-
-	// Finally the document is closed and the main thread's
-	// context is freed.
-
-	fz_drop_document(ctx, doc);
-	fz_drop_context(ctx);
-
 	return 0;
 }
